#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@Author        : EINDEX Li
@File          : workers.py
@Created       : 22/12/2017
"""
import asyncio

# print(workers_mapping)

response_queue_names = []


async def start():
    print('Start')
    print('End')
    while True:
        await asyncio.sleep(1)


if __name__ == '__main__':
    loop = asyncio.get_event_loop()
    loop.run_until_complete(start())
    loop.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@Author        : EINDEX Li
@File          : dict_tools.py
@Created       : 22/12/2017
"""


def get_info(dic: dict or list, parser, default=None):
    temp = dic
    for index, token in enumerate(parser.split('.')):
        if isinstance(temp, list):
            unused_parsing = '.'.join(parser.split('.')[index:])
            return [get_info(t, unused_parsing) for t in temp]
        if temp.get(token):
            temp = temp.get(token)
        elif temp.get(token) in [False, dict(), list()]:
            return temp.get(token)
        else:
            return default
    return temp



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@Author        : EINDEX Li
@File          : config.py
@Created       : 21/12/2017
"""
from aiospider.tools.singleton import Singleton


class Config(metaclass=Singleton):

    redis_conn = 'redis://localhost:6379/2'




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@Author        : EINDEX Li
@File          : job.py
@Created       : 21/12/2017
"""
import json

from aiospider.tools.job_queue import JobQueue
from aiospider.tools.redis_pool import RedisPool


class RequestJob:
    def __init__(self,
                 worker,
                 url,
                 method='GET',
                 params=None,
                 data=None,
                 keys=None,
                 identity=None,
                 headers=None,
                 cookies=None,
                 proxy=None,
                 name=None,
                 allow_redirect=None,
                 redirect_times=None):
        self.url = url
        self.keys = keys
        self.params = params
        self.data = data
        self.method = method
        self.headers = headers
        self.cookies = cookies
        self.proxy = proxy
        self.allow_redirect = allow_redirect
        self.redirect_times = redirect_times
        self.identity = identity
        self.worker = worker
        self.name = name

    def to_dict(self):
        content = {}
        for k, v in self.__dict__.items():
            if v:
                content[k] = v
        return content

    def to_json(self):
        return json.dumps(self.to_dict())

    @staticmethod
    async def bloom_filter(name, worker, url, params=None, **kwargs):
        pool = await RedisPool.get_pool().pool
        if params:
            key = f'{url}{sorted(params.items())}'
        else:
            key = f'{url}'
        return await pool.execute('sismember', f'{name}:{worker}:filter', key)

    async def send(self):
        content = {}
        for k, v in self.__dict__.items():
            if v:
                content[k] = v
        if not await self.bloom_filter(**content):
            queue = await JobQueue.get_queue(f'{self.name}:{self.worker}:request')
            await queue.push(json.dumps(content))
            return True
        else:
            return False

    def get_request_params(self):
        return {
            'url': self.url,
            'params': self.params,
            'headers': self.headers,
            'method': self.method,
            'proxy': self.proxy,
            'data': self.data,
            'allow_redirects': self.allow_redirect,
            'max_redirects': self.redirect_times,
        }

    @staticmethod
    def from_json(content):
        return RequestJob(**json.loads(content))

    def __repr__(self):
        return self.to_json()


class ResponseJob:
    def __init__(self,
                 request_job: dict or RequestJob,
                 url,
                 worker,
                 name,
                 success,
                 identity=None,
                 content=None,
                 content_type=None,
                 status_code=None,
                 cookies=None,
                 proxy=None):
        if isinstance(request_job, dict):
            self.request_job = RequestJob(**request_job)
        elif isinstance(request_job, RequestJob):
            self.request_job = request_job
        self.url = url
        self.identity = identity
        self.worker = worker
        self.name = name
        self.status_code = status_code
        self.content = content
        self.cookies = cookies
        self.content_type = content_type
        self.success = success
        self.proxy = proxy

    def to_dict(self):
        content = {}
        for k, v in self.__dict__.items():
            if v:
                if isinstance(v, RequestJob):
                    v = v.to_dict()
                content[k] = v
        return content

    def to_json(self):
        return json.dumps(self.to_dict())

    async def send(self):
        queue = await JobQueue.get_queue(f'{self.name}:{self.worker}:response')
        await queue.push(self.to_json())

    @staticmethod
    def from_json(content):
        return ResponseJob(**json.loads(content))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@Author        : EINDEX Li
@File          : singleton.py
@Created       : 21/12/2017
"""


class Singleton(type):
    def __init__(cls, name, bases, attrs):
        super(Singleton, cls).__init__(name, bases, attrs)
        cls._instance = {}

    def __call__(cls, *args, **kwargs):
        key = cls.key_gen(*args, **kwargs)
        cls._instance.setdefault(key, None)
        if cls._instance[key] is None:
            cls._instance[key] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instance[key]

    @staticmethod
    def key_gen(*args, **kwargs):
        return f'{sorted(args)}:{sorted(kwargs.items())}'


class Foo(metaclass=Singleton):
    def __init__(self, x):
        self.x = x


if __name__ == '__main__':
    a = Foo(1)
    print(a.x)
    b = Foo(2)
    print(a.x)
    c = Foo(1)
    print(b.x)

    print(a is c)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@Author        : EINDEX Li
@File          : redis_pool.py
@Created       : 21/12/2017
"""
import asyncio

import aioredis

from aiospider import config
from aiospider.tools.singleton import Singleton


class RedisPool(metaclass=Singleton):
    def __init__(self, redis_url):
        self.redis_url = redis_url
        self._pool = None

    @property
    async def pool(self):
        if not self._pool:
            self._pool = await aioredis.create_connection(self.redis_url)
        return self._pool

    @classmethod
    def get_pool(cls):
        return RedisPool(redis_url=config.Config.redis_conn)


if __name__ == '__main__':
    async def test():
        x = RedisPool.get_pool()
        x1 = RedisPool.get_pool()
        x2 = RedisPool.get_pool()
        x3 = RedisPool.get_pool()
        x4 = RedisPool.get_pool()
        x5 = RedisPool.get_pool()
        print(await x.pool)
        print(await x1.pool)
        print(await x2.pool)
        print(await x3.pool)
        print(await x4.pool)
        print(await x5.pool)
        x5.pool.close()
        await asyncio.sleep(1)


    loop = asyncio.get_event_loop()
    loop.run_until_complete(test())
    loop.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@Author        : EINDEX Li
@File          : request.py
@Created       : 21/12/2017
"""
import logging

import aiohttp

from aiospider.models.job import ResponseJob, RequestJob
from aiospider.tools.job_queue import JobQueue
from aiospider.tools.redis_pool import RedisPool


class Bot:
    def __init__(self, bot_id=None, headers=None, cookies=None, proxy=None, status=False, loop=None):
        self.bot_id = bot_id
        self._session = None
        self.headers = headers
        self.cookies = cookies
        self._proxy = proxy
        self.loop = loop
        self.status = False

    @property
    def session(self):
        if not self._session:
            return aiohttp.ClientSession(headers=self.headers, cookies=self.cookies, loop=self.loop)
        return self._session

    async def __aenter__(self):
        return self

    def __enter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        return self.close()

    def __exit__(self, exc_type, exc_val, exc_tb):
        return self.close()

    def close(self):
        if self._session:
            self._session.close()
        return True

    @staticmethod
    async def request(session, *args, **kwargs):
        async with session.request(*args, **kwargs) as resp:
            return resp.status, await resp.text()

    async def start(self, name):
        try:
            self.status = True
            queue = await JobQueue.get_queue(name)
            async with self.session as session:
                while True:
                    item = await queue.get()
                    if item:
                        item = item.decode()
                        try:
                            req_job = RequestJob.from_json(item)
                            # pool = await RedisPool.get_pool().pool
                            # if req_job.params:
                            #     key = f'{req_job.url}{sorted(req_job.params.items())}'
                            # else:
                            #     key = f'{req_job.url}'
                            # if await pool.execute('sismember', f'{req_job.name}:{req_job.worker}:filter', key):
                            #     continue
                        except Exception as e:
                            logging.exception(e)
                            continue
                        try:
                            status, content = await self.request(session, **req_job.get_request_params())
                            print(status, content)
                            await ResponseJob(request_job=req_job, url=req_job.url, worker=req_job.worker,
                                              name=req_job.name,
                                              success=True,
                                              status_code=status,
                                              content=content).send()
                        except Exception as e:
                            await ResponseJob(request_job=req_job, url=req_job.url, worker=req_job.worker,
                                              name=req_job.name,
                                              success=True,
                                              status_code=400,
                                              content='{}').send()
                            logging.exception(e)
        except Exception as e:
            logging.exception(e)
            self.status = False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

setup(
        name='aiospider',
        version='0.0.4.11',
        description='A spider use asyncio.',
        license='MIT License',
        install_requires=['cchardet', 'aiodns', 'aiohttp', 'async-timeout', 'aioredis'],

        author='EINDEX',
        author_email='hi@eindex.me',

        packages=find_packages(),
        platforms='any',
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@Author        : EINDEX Li
@File          : clawer.py
@Created       : 22/12/2017
"""
import asyncio

from aiospider.request.request import Bot
from aiospider.tools.redis_pool import RedisPool


async def start():
    bot_dict = {}
    # conn = (await RedisPool.get_pool()).pool
    while True:
        conn = await RedisPool.get_pool().pool
        accounts = await conn.execute('smembers', 'account_token')
        for account in accounts:
            account = account.decode()
            if not bot_dict.get(account):
                bot_dict[account] = Bot(headers={'Authorization': account})
                asyncio.gather(bot_dict[account].start('zhihu:request'))
        await asyncio.sleep(1)
        print(bot_dict)


if __name__ == '__main__':
    loop = asyncio.get_event_loop()
    loop.run_until_complete(start())
    loop.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@Author        : EINDEX Li
@File          : base_worker.py
@Created       : 22/12/2017
"""
import asyncio
import logging

from aiospider.models.job import RequestJob, ResponseJob
from aiospider.tools.job_queue import JobQueue


class Worker:
    keys = {}
    name = None
    worker = None
    url_regex = None
    method = None
    params = {}
    require_params = []
    headers = {}

    @classmethod
    async def analyze(cls, request_job: RequestJob, content):
        pass

    @staticmethod
    def parser(data: dict):
        pass

    @staticmethod
    def pre_handle(resp_job: ResponseJob):
        return True

    @classmethod
    async def _per_handle(cls, resp_job: ResponseJob):
        if cls.pre_handle(resp_job):
            return await cls.analyze(request_job=resp_job.request_job, content=resp_job.content)
        else:
            raise ValueError()

    @classmethod
    def request_builder(cls, keys: dict, params=None, headers=None, data=None, identity=None,
                        redirect_times=None,
                        allow_redirect=None,
                        proxy=None,
                        cookies=None):
        url = cls.url_regex
        for key, value in keys.items():
            url = cls.url_regex.replace('{%s}' % key, str(value))
        for must in cls.require_params:
            if must not in params:
                raise ValueError()
        if isinstance(params, dict):
            cls.params.update(params)
        if headers:
            cls.headers.update(headers)
        return RequestJob(worker=cls.worker, url=url, method=cls.method, params=cls.params, data=data,
                          identity=identity, headers=cls.headers,
                          keys=keys,
                          cookies=cookies, proxy=proxy, name=cls.name, allow_redirect=allow_redirect,
                          redirect_times=redirect_times)

    @classmethod
    async def start(cls):
        print(f'{cls.name}:{cls.worker}:Start')
        while True:
            try:
                queue = await JobQueue.get_queue(f'{cls.name}:{cls.worker}:response')
                content = await queue.get()
                if content:
                    resp_job = ResponseJob.from_json(content.decode())
                    await cls._per_handle(resp_job)
                else:
                    await asyncio.sleep(1)

            except Exception as e:
                logging.exception(e)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@Author        : EINDEX Li
@File          : queue.py
@Created       : 21/12/2017
"""
import asyncio

from aiospider.tools.redis_pool import RedisPool
from aiospider.tools.singleton import Singleton


class JobQueue(metaclass=Singleton):
    def __init__(self, name, pool):
        self.name = name
        self.pool = pool

    @staticmethod
    async def get_queue(name):
        pool = await RedisPool.get_pool().pool
        return JobQueue(name=name, pool=pool)

    async def push(self, *items, reverse=False):
        # calling methods of Redis class
        command = f'{"r" if reverse else "l"}push'
        return await self.pool.execute(command, self.name, *items)

    async def get(self, reverse=False):
        command = f'{"l" if reverse else "r"}pop'
        return await self.pool.execute(command, self.name)


if __name__ == '__main__':
    async def test():
        x = await JobQueue.get_queue('test')
        x1 = await JobQueue.get_queue('test')
        print(x)
        print(x1)
        print(x is x1)
        await x.push('1')
        print(await x.get())


    loop = asyncio.get_event_loop()
    loop.run_until_complete(test())
    loop.close()
